@@ -52,8 +52,6 @@ type Service interface {
 	Ps(ctx context.Context, projectName string, options PsOptions) ([]ContainerSummary, error)
 	// List executes the equivalent to a `docker stack ls`
 	List(ctx context.Context, options ListOptions) ([]Stack, error)
-	// Config executes the equivalent to a `compose config`
-	Config(ctx context.Context, project *types.Project, options ConfigOptions) ([]byte, error)
 	// Kill executes the equivalent to a `compose kill`
 	Kill(ctx context.Context, projectName string, options KillOptions) error
 	// RunOneOffContainer creates a service oneoff container and starts its dependencies
@@ -116,9 +114,13 @@ type VizOptions struct {
 	Indentation string
 }
 
+// WatchLogger is a reserved name to log watch events
+const WatchLogger = "#watch"
+
 // WatchOptions group options of the Watch API
 type WatchOptions struct {
-	Build BuildOptions
+	Build *BuildOptions
+	LogTo LogConsumer
 }
 
 // BuildOptions group options of the Build API
@@ -207,17 +209,27 @@ type StartOptions struct {
 	Attach LogConsumer
 	// AttachTo set the services to attach to
 	AttachTo []string
-	// CascadeStop stops the application when a container stops
-	CascadeStop bool
+	// OnExit defines behavior when a container stops
+	OnExit Cascade
 	// ExitCodeFrom return exit code from specified service
 	ExitCodeFrom string
 	// Wait won't return until containers reached the running|healthy state
 	Wait        bool
 	WaitTimeout time.Duration
 	// Services passed in the command line to be started
-	Services []string
+	Services       []string
+	Watch          bool
+	NavigationMenu bool
 }
 
+type Cascade int
+
+const (
+	CascadeIgnore Cascade = iota
+	CascadeStop   Cascade = iota
+	CascadeFail   Cascade = iota
+)
+
 // RestartOptions group options of the Restart API
 type RestartOptions struct {
 	// Project is the compose project used to define this app. Might be nil if user ran command just with project name
@@ -300,6 +312,8 @@ type KillOptions struct {
 	Services []string
 	// Signal to send to containers
 	Signal string
+	// All can be set to true to try to kill all found containers, independently of their state
+	All bool
 }
 
 // RemoveOptions group options of the Remove API
