@@ -26,7 +26,7 @@ import (
 // logPrinter watch application containers an collect their logs
 type logPrinter interface {
 	HandleEvent(event api.ContainerEvent)
-	Run(cascadeStop bool, exitCodeFrom string, stopFn func() error) (int, error)
+	Run(cascade api.Cascade, exitCodeFrom string, stopFn func() error) (int, error)
 	Cancel()
 	Stop()
 }
@@ -79,14 +79,15 @@ func (p *printer) HandleEvent(event api.ContainerEvent) {
 }
 
 //nolint:gocyclo
-func (p *printer) Run(cascadeStop bool, exitCodeFrom string, stopFn func() error) (int, error) {
+func (p *printer) Run(cascade api.Cascade, exitCodeFrom string, stopFn func() error) (int, error) {
 	var (
 		aborting bool
 		exitCode int
 	)
 	defer p.Stop()
 
-	containers := map[string]struct{}{}
+	// containers we are tracking. Use true when container is running, false after we receive a stop|die signal
+	containers := map[string]bool{}
 	for {
 		select {
 		case <-p.stopCh:
@@ -100,19 +101,21 @@ func (p *printer) Run(cascadeStop bool, exitCodeFrom string, stopFn func() error
 				if _, ok := containers[id]; ok {
 					continue
 				}
-				containers[id] = struct{}{}
+				containers[id] = true
 				p.consumer.Register(container)
 			case api.ContainerEventExit, api.ContainerEventStopped, api.ContainerEventRecreated:
-				if !event.Restarting {
-					delete(containers, id)
-				}
-				if !aborting {
+				if !aborting && containers[id] {
 					p.consumer.Status(container, fmt.Sprintf("exited with code %d", event.ExitCode))
 					if event.Type == api.ContainerEventRecreated {
 						p.consumer.Status(container, "has been recreated")
 					}
 				}
-				if cascadeStop {
+				containers[id] = false
+				if !event.Restarting {
+					delete(containers, id)
+				}
+
+				if cascade == api.CascadeStop {
 					if !aborting {
 						aborting = true
 						err := stopFn()
@@ -120,14 +123,26 @@ func (p *printer) Run(cascadeStop bool, exitCodeFrom string, stopFn func() error
 							return 0, err
 						}
 					}
-					if event.Type == api.ContainerEventExit {
-						if exitCodeFrom == "" {
-							exitCodeFrom = event.Service
-						}
-						if exitCodeFrom == event.Service {
-							exitCode = event.ExitCode
+				}
+				if event.Type == api.ContainerEventExit {
+					if cascade == api.CascadeFail && event.ExitCode != 0 {
+						exitCodeFrom = event.Service
+						if !aborting {
+							aborting = true
+							err := stopFn()
+							if err != nil {
+								return 0, err
+							}
 						}
 					}
+					if cascade == api.CascadeStop && exitCodeFrom == "" {
+						exitCodeFrom = event.Service
+					}
+				}
+
+				if exitCodeFrom == event.Service && (event.Type == api.ContainerEventExit || event.Type == api.ContainerEventStopped) {
+					// Container was interrupted or exited, let's capture exit code
+					exitCode = event.ExitCode
 				}
 				if len(containers) == 0 {
 					// Last container terminated, done
